import os
import sys
import json
import subprocess

from heapq import heappush

import yaml
import six

from .config import load_config_file
from .args import parse_args
from .env import compile


def main():
    args = parse_args()

    actions = load_config_file()
    actions.extend(args.actions or [])

    os.environ = compile(actions)

    if args.cmd:
        sys.exit(subprocess.call(' '.join(args.cmd), shell=True))
    else:
        # print our env as a file sourceable in bash.
        items = sorted([(k, v) for k, v in six.iteritems(os.environ)])
        for k, v in items:
            print("export %s='%s'" % (k, v))

if __name__ == '__main__':
    main()



import os
import six


def flatten_list(node, prefix=None):
    for v in node:
        for kid in flatten_dict(v, prefix):
            yield kid


def flatten_dict(node, prefix=None):
    """
    This is an iterator that returns a list of flattened env
    vars based on the conf file supplied
    """
    for k, v in six.iteritems(node):
        if prefix:
            k = '%s_%s' % (prefix, k)

        # We have a value we can stringify
        if not isinstance(v, (dict, list)):
            yield (k, os.path.expandvars(str(v)))
        else:
            for kid in flatten(v, prefix=k):
                yield kid


def flatten(node, prefix=None):
    flat_func = flatten_dict
    if isinstance(node, list):
        flat_func = flatten_list

    for kid in flat_func(node, prefix):
        yield kid



"""
Compile our environment from directories and files.
"""
import os

from heapq import heappush

import yaml

from withenv.flatten import flatten


def find_yml_in_dir(dirname):
    def is_yaml(fn):
        return fn.endswith(('yml', 'yaml'))

    fnames = []  # a heap

    for dirpath, dirnames, filenames in os.walk(dirname):
        for fn in filter(is_yaml, filenames):
            heappush(fnames, os.path.join(dirpath, fn))

    return fnames


def update_env_from_dir(dirname, env):
    for fname in find_yml_in_dir(dirname):
        update_env_from_file(fname, env)


def update_env_from_file(fname, env):
    new_env = yaml.safe_load(open(fname))
    flat_env = {}

    # Order isn't important
    if isinstance(new_env, dict):
        new_env = [new_env]

    for item in new_env:
        for k, v in flatten(item):
            env[k] = v


def update_env_from_alias(fname, env):
    action_list = yaml.safe_load(open(fname))
    actions = [
        (k, v) for action in action_list
        for k, v in action.items()
    ]
    return compile(actions, env)


def find_action(name):
    actions = {
        'file': update_env_from_file,
        'directory': update_env_from_dir,
        'alias': update_env_from_alias,
    }
    return actions[name]


def compile(actions=None, env=None):
    actions = actions or []
    env = env if env is not None else os.environ

    for action, arg in actions:
        find_action(action)(arg, env)
    return env



import sys
import textwrap

from argparse import ArgumentParser, Action, REMAINDER


class AddToEnvsAction(Action):

    action_map = {
        '-e': 'file',
        '--env': 'file',
        '-d': 'directory',
        '--dir': 'directory',
        '-a': 'alias',
        '--alias': 'alias',
    }

    def __init__(self, *args, **kw):
        super(AddToEnvsAction, self).__init__(*args, **kw)
        self.env_type = self.action_map.get(self.dest)

    def __call__(self, parser, namespace, values, option_string):
        envs = getattr(namespace, self.dest) or []
        envs.append((self.action_map[option_string], values))
        setattr(namespace, self.dest, envs)


def parse_args(args=None):
    actions = []
    parser = ArgumentParser(
        prog='we',
        usage='%(prog)s -h [-e ENV_YAML] [-d DIR] [-a ALIAS_YAML] CMD',
        description=('Prepare the environment variables '
                     'prior to running a command.'),

        epilog=('More than one flag can be used a time. '
                'Each flag will be applied to the environment '
                'variables in order, allowing a cascade of changes.')
    )

    parser.add_argument(
        '-e', '--env', dest='actions',
        nargs='?', action=AddToEnvsAction,
        help='a YAML file to include in the environment',
        metavar='YML',
    )

    parser.add_argument(
        '-d', '--dir', dest='actions',
        nargs='?', action=AddToEnvsAction,
        help=('a directory containing YAML files to '
              'recursively apply to the environment'),
        metavar='DIR',
    )

    parser.add_argument(
        '-a', '--alias', dest='actions',
        nargs='?', action=AddToEnvsAction,
        help=('a YAML file containing a list of '
              'file/directory to apply to the environment'),
        metavar='ALIAS YML',
    )

    parser.add_argument(
        'cmd', nargs=REMAINDER,
        help='The command to run with the supplied environment.',
        metavar='CMD'
    )

    args = args if args is not None else sys.argv[1:]

    return parser.parse_args(args)



#!/usr/bin/env python
# -*- coding: utf-8 -*-


try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


readme = open('README.rst').read()

requirements = [
    'PyYAML',
    'six',
]

setup(
    name='withenv',
    version='0.5.0',
    description=('Prefix commands with specific environments specified '
                 'in YAML files.'),
    long_description=readme,
    author='Eric Larson',
    author_email='eric@ionrock.org',
    url='https://github.com/ionrock/withenv',
    packages=[
        'withenv',
    ],
    package_dir={'withenv':
                 'withenv'},
    include_package_data=True,
    install_requires=requirements,
    license="BSD",
    zip_safe=False,
    keywords='withenv',
    entry_points={
        'console_scripts': [
            'we = withenv.cli:main',
        ]
    },
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
    ],
)



import os
import yaml


CONFIG_FILE = '.werc'


def find_config_file(start=None, filename=CONFIG_FILE):
    start = start or os.getcwd()
    conf = os.path.join(start, filename)
    if os.path.exists(conf):
        return conf

    if start == '/':
        return None

    parent = os.path.normpath(os.path.abspath(os.path.join(start, '..')))
    return find_config_file(parent)


def load_config_file():
    config_path = find_config_file()
    if not config_path:
        return []
    return [('alias', config_path)]
